package geometry.part2;

import geometry.part1.Line;

import java.awt.geom.Point2D;

/**
 * 2点(x1, y1) - (x2, y2)を結ぶ線分を表現するクラス
 */
public class LineSegment {
    public double x1;
    public double y1;
    public double x2;
    public double y2;

    public LineSegment(double x1, double y1, double x2, double y2) {
        this.x1 = x1;
        this.y1 = y1;
        this.x2 = x2;
        this.y2 = y2;
    }

    public Line toLine() {
        return Line.fromPoints(x1, y1, x2, y2);
    }

    public boolean intersects(Line l) {
        double t1 = l.a * x1 + l.b * y1 + l.c;
        double t2 = l.a * x2 + l.b * y2 + l.c;
        return t1 * t2 <= 0;
    }

    public boolean intersects(LineSegment s) {
        return intersects(s.toLine()) && s.intersects(toLine());
    }

    public Point2D getIntersectionPoint(Line l) {
        if (!intersects(l)) {
            return null; // 交差しない場合はnullを返す
        }
        return l.getIntersectionPoint(toLine());
    }

    public Point2D getIntersectionPoint(LineSegment s) {
        if (!intersects(s)) {
            return null; // 交差しない場合はnullを返す
        }
        return s.toLine().getIntersectionPoint(toLine());
    }

    @Override
    public String toString() {
        return "(" + x1 + ", " + y1 + ") - (" + x2 + ", " + y2 + ")";
    }
}
